<?php
namespace OCA\Dematpayslip\Settings;

use OCP\AppFramework\Http\TemplateResponse;
use OCP\BackgroundJob\IJobList;
use OCP\IConfig;
use OCP\IDateTimeFormatter;
use OCP\IL10N;
use OCP\Settings\ISettings;

use OCP\Files\Folder;
use OCA\Dematpayslip\Db\DematpayslipUserMapper;

class AdminSettings implements ISettings {
    /** @var IConfig */
    private $config;

    /** @var IL10N */
    private $l;

    /** @var IDateTimeFormatter */
    private $dateTimeFormatter;

    /** @var IJobList */
    private $jobList;

    private $appName;

    /** @var string User Id */
    private $userId;

    /** @var DematpayslipUserMapper*/
    private $_dematpayslipUserMapper;

    // dematpayslip config
    private $_dematpayslipConfigList = array();

    /** @var array Message List */
    private $_msgList = array(
        'error'   => array(),
        'success' => array(),
        'warning' => array(),
    );

    /**
     * Assign params for template
     *
     * @return  array   Params for template
     */
    private function _assignParams() {
        $this->_dematpayslipConfigList['payslip_dir']  = \OC::$server->getConfig()->getAppValue($this->appName, 'payslip_dir', 'Bulletins'); // Bulletins
        $this->_dematpayslipConfigList['user_id']      = \OC::$server->getConfig()->getAppValue($this->appName, 'user_id', 'demat'); // demat user
        $this->_dematpayslipConfigList['app_data_dir'] = \OC::$server->getConfig()->getAppValue($this->appName, 'app_data_dir', 'dematpayslip'); // dematpayslip directory of user demat (upload, check, report, etc)

        return array(
            'dematpayslipConfigList' => $this->_dematpayslipConfigList,
            'msgList' => $this->_msgList,
        );
    }

    /**
     * Get payslip share list
     *
     * @return  array|\OCP\Share\IShare[][]
     * @throws  \OCP\Files\NotFoundException
     */
    private function _payslipShareList() {
        $payslipShareList = array();

        $payslipDir = \OC::$server->getConfig()->getAppValue($this->appName, 'payslip_dir');
        $payslipUserId = \OC::$server->getConfig()->getAppValue($this->appName, 'user_id');

        $rootFolder   = \OC::$server->getRootFolder();
        $shareManager = \OC::$server->getShareManager();
        $userFolder   = $rootFolder->getUserFolder($payslipUserId);

        $payslipShareNode = $userFolder->get($payslipDir);
        if ($payslipShareNode instanceof Folder) {
            $payslipShareList = $shareManager->getSharesInFolder($payslipUserId, $payslipShareNode);
        }

        return $payslipShareList;
    }

    /**
     * Reshare all payslips
     *
     * @throws  \OCP\Files\InvalidPathException
     * @throws  \OCP\Files\NotFoundException
     */
    private function _payslipsReshare() {
        $payslipDir = \OC::$server->getConfig()->getAppValue($this->appName, 'payslip_dir');
        $payslipUserId = \OC::$server->getConfig()->getAppValue($this->appName, 'user_id');

        $rootFolder   = \OC::$server->getRootFolder();
        $shareManager = \OC::$server->getShareManager();
        $userFolder   = $rootFolder->getUserFolder($payslipUserId);
        $userManager  = \OC::$server->getUserManager();

        // payslip already shared
        $payslipShareList = $this->_payslipShareList();
        $payslipMatriculeShareNodeIdList = array();
        if (count($payslipShareList) > 0) {
            foreach ($payslipShareList as $payslipShareNodeId => $payslipShareArr) {
                $payslipMatriculeShareNodeIdList[] = $payslipShareNodeId;
            }
        }

        $payslipFolder = $userFolder->get($payslipDir);
        if ($payslipFolder instanceof Folder) {
            $payslipMatriculeNodeList = $payslipFolder->getDirectoryListing();
            foreach ($payslipMatriculeNodeList as $payslipMatriculeNode) {
                if ($payslipMatriculeNode instanceof Folder) {
                    $siret = '';
                    $matricule = '';
                    $siretMatricule = $payslipMatriculeNode->getName();
                    $siretMatriculeArr = explode('-', $siretMatricule);
                    if (count($siretMatriculeArr)>=2) {
                        $siret     = $siretMatriculeArr[0];
                        $matricule = $siretMatriculeArr[1];
                    }

                    $dematpayslipUserList = $this->_dematpayslipUserMapper->findAllBySiretAndMatricule($siret, $matricule);
                    if (count($dematpayslipUserList) === 1) {
                        $dematpayslipUser = $dematpayslipUserList[0];

                        // s'il l'utilisateur n'a pas desactive la dematerialisation de son bulletin de paie
                        //if ($dematpayslipUser->getDisabled() != 1) {
                            // verifier que l'utilisateur existe toujours
                            $dematpayslipUserExists = $userManager->userExists($dematpayslipUser->getUserId());
                            if ($dematpayslipUserExists === true) {
                                // verifier si le dossier n'est pas deja partage
                                if (!in_array($payslipMatriculeNode->getId(), $payslipMatriculeShareNodeIdList)) {
                                    // reshare
                                    $payslipMatriculeShare = $shareManager->newShare();
                                    $payslipMatriculeShare->setNode($payslipMatriculeNode);
                                    $payslipMatriculeShare->setShareType(\OC\Share\Share::SHARE_TYPE_USER);
                                    $payslipMatriculeShare->setSharedWith($dematpayslipUser->getUserId());
                                    $payslipMatriculeShare->setPermissions(\OCP\Constants::PERMISSION_READ);
                                    $payslipMatriculeShare->setSharedBy($payslipUserId);
                                    $payslipMatriculeShare = $shareManager->createShare($payslipMatriculeShare);
                                }
                            }
                        //}
                    } else {
                        // error
                        $this->_msgList['error'][] = $this->l->t('Impossible de re-partager le dossier [matricule="' . $matricule . '"] : aucun utilisateur n\'est associé.');
                        break;
                    }
                }
            }
        }
    }

    /**
     * Unshare all payslips
     *
     * @throws  \OCP\Files\NotFoundException
     * @throws  \OCP\Share\Exceptions\ShareNotFound
     */
    private function _payslipsUnshare() {
        $payslipShareList = self::_payslipShareList();

        $shareManager = \OC::$server->getShareManager();
        if (count($payslipShareList) > 0) {
            foreach ($payslipShareList as $payslipShareArr) {
                if (is_array($payslipShareArr) && count($payslipShareArr)>=1) {
                    $shareManager->deleteShare($payslipShareArr[0]);
                }
            }
        }
    }

    /**
     * AdminSettings constructor.
     *
     * @param   string                  $AppName                    App name
     * @param   IConfig                 $config                     Config
     * @param   IL10N                   $l                          Lang
     * @param   IDateTimeFormatter      $dateTimeFormatter          Date formatter
     * @param   IJobList                $jobList                    Job list
     * @param   DematpayslipUserMapper  $dematpayslipUserMapper     Dematpayslip user mapper
     * @param   string                  $UserId                     User Id
     */
    public function __construct($AppName,
                                IConfig $config,
                                IL10N $l,
                                IDateTimeFormatter $dateTimeFormatter,
                                IJobList $jobList,
                                DematpayslipUserMapper $dematpayslipUserMapper,
                                $UserId
    ) {
        $this->appName = $AppName;
        $this->config = $config;
        $this->l = $l;
        $this->dateTimeFormatter = $dateTimeFormatter;
        $this->jobList = $jobList;
        $this->_dematpayslipUserMapper = $dematpayslipUserMapper;
        $this->userId = $UserId;

        $this->_assignParams();
    }

    /**
     * Get form
     *
     * @return  TemplateResponse
     *
     * @throws  \OCP\Files\InvalidPathException
     * @throws  \OCP\Files\NotFoundException
     * @throws  \OCP\Share\Exceptions\ShareNotFound
     */
    public function getForm() {
        $action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_STRING);

        if ($action === 'payslips_config') {
            $dematpayslipDir = filter_input(INPUT_GET, 'dematpayslip_payslip_dir', FILTER_SANITIZE_STRING);
            $dematpayslipUserUserId = filter_input(INPUT_GET, 'dematpayslip_user_user_id', FILTER_SANITIZE_STRING);
            $dematpayslipAppDataDir = filter_input(INPUT_GET, 'dematpayslip_app_data_dir', FILTER_SANITIZE_STRING);
            \OC::$server->getConfig()->setAppValue($this->appName, 'payslip_dir', $dematpayslipDir);
            \OC::$server->getConfig()->setAppValue($this->appName, 'user_id', $dematpayslipUserUserId);
            \OC::$server->getConfig()->setAppValue($this->appName, 'app_data_dir', $dematpayslipAppDataDir);
        } else if ($action === 'payslips_unshare') {
            $this->_payslipsUnshare();
        } else if ($action === 'payslips_reshare') {
            $this->_payslipsReshare();
        }

        if (!empty($action)) {
            if (count($this->_msgList['error']) <= 0) {
                $this->_msgList['success'][] = $this->l->t('Opération réussie.');
            }
        }

        return new TemplateResponse('dematpayslip', 'settings/admin', $this->_assignParams());
    }

    /**
     * @return string the section ID, e.g. 'sharing'
     */
    public function getSection() {
        return 'dematpayslip';
    }

    /**
     * @return int whether the form should be rather on the top or bottom of
     * the admin section. The forms are arranged in ascending order of the
     * priority values. It is required to return a value between 0 and 100.
     */
    public function getPriority() {
        return 50;
    }
}